# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Any, Optional

# pylint: disable=R0902,too-many-locals
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    ImageModelSettingsClassification as RestImageModelSettingsClassification,
)
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    ImageModelSettingsObjectDetection as RestImageModelSettingsObjectDetection,
)
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    LearningRateScheduler,
    LogTrainingMetrics,
    LogValidationLoss,
    ModelSize,
    StochasticOptimizer,
    ValidationMetricType,
)
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class ImageModelDistributionSettings(RestTranslatableMixin):
    """Model settings for all AutoML Image Verticals.
    Please do not instantiate directly. Use the child classes instead.

    :param advanced_settings: Settings for advanced scenarios.
    :type advanced_settings: str
    :param ams_gradient: Enable AMSGrad when optimizer is 'adam' or 'adamw'.
    :type ams_gradient: bool
    :param beta1: Value of 'beta1' when optimizer is 'adam' or 'adamw'. Must be a float in the range
     [0, 1].
    :type beta1: float
    :param beta2: Value of 'beta2' when optimizer is 'adam' or 'adamw'. Must be a float in the range
     [0, 1].
    :type beta2: float
    :param checkpoint_frequency: Frequency to store model checkpoints. Must be a positive integer.
    :type checkpoint_frequency: int
    :param checkpoint_run_id: The id of a previous run that has a pretrained checkpoint for
     incremental training.
    :type checkpoint_run_id: str
    :param distributed: Whether to use distributed training.
    :type distributed: bool
    :param early_stopping: Enable early stopping logic during training.
    :type early_stopping: bool
    :param early_stopping_delay: Minimum number of epochs or validation evaluations to wait before
     primary metric improvement
     is tracked for early stopping. Must be a positive integer.
    :type early_stopping_delay: int
    :param early_stopping_patience: Minimum number of epochs or validation evaluations with no
     primary metric improvement before
     the run is stopped. Must be a positive integer.
    :type early_stopping_patience: int
    :param enable_onnx_normalization: Enable normalization when exporting ONNX model.
    :type enable_onnx_normalization: bool
    :param evaluation_frequency: Frequency to evaluate validation dataset to get metric scores. Must
     be a positive integer.
    :type evaluation_frequency: int
    :param gradient_accumulation_step: Gradient accumulation means running a configured number of
     "GradAccumulationStep" steps without
     updating the model weights while accumulating the gradients of those steps, and then using
     the accumulated gradients to compute the weight updates. Must be a positive integer.
    :type gradient_accumulation_step: int
    :param layers_to_freeze: Number of layers to freeze for the model. Must be a positive integer.
     For instance, passing 2 as value for 'seresnext' means
     freezing layer0 and layer1. For a full list of models supported and details on layer freeze,
     please
     see: https://learn.microsoft.com/azure/machine-learning/how-to-auto-train-image-models.
    :type layers_to_freeze: int
    :param learning_rate: Initial learning rate. Must be a float in the range [0, 1].
    :type learning_rate: float
    :param learning_rate_scheduler: Type of learning rate scheduler. Must be 'warmup_cosine' or
     'step'. Possible values include: "None", "WarmupCosine", "Step".
    :type learning_rate_scheduler: str or
     ~azure.mgmt.machinelearningservices.models.LearningRateScheduler
    :param model_name: Name of the model to use for training.
     For more information on the available models please visit the official documentation:
     https://learn.microsoft.com/azure/machine-learning/how-to-auto-train-image-models.
    :type model_name: str
    :param momentum: Value of momentum when optimizer is 'sgd'. Must be a float in the range [0, 1].
    :type momentum: float
    :param nesterov: Enable nesterov when optimizer is 'sgd'.
    :type nesterov: bool
    :param number_of_epochs: Number of training epochs. Must be a positive integer.
    :type number_of_epochs: int
    :param number_of_workers: Number of data loader workers. Must be a non-negative integer.
    :type number_of_workers: int
    :param optimizer: Type of optimizer. Possible values include: "None", "Sgd", "Adam", "Adamw".
    :type optimizer: str or ~azure.mgmt.machinelearningservices.models.StochasticOptimizer
    :param random_seed: Random seed to be used when using deterministic training.
    :type random_seed: int
    :param step_lr_gamma: Value of gamma when learning rate scheduler is 'step'. Must be a float in
     the range [0, 1].
    :type step_lr_gamma: float
    :param step_lr_step_size: Value of step size when learning rate scheduler is 'step'. Must be a
     positive integer.
    :type step_lr_step_size: int
    :param training_batch_size: Training batch size. Must be a positive integer.
    :type training_batch_size: int
    :param validation_batch_size: Validation batch size. Must be a positive integer.
    :type validation_batch_size: int
    :param warmup_cosine_lr_cycles: Value of cosine cycle when learning rate scheduler is
     'warmup_cosine'. Must be a float in the range [0, 1].
    :type warmup_cosine_lr_cycles: float
    :param warmup_cosine_lr_warmup_epochs: Value of warmup epochs when learning rate scheduler is
     'warmup_cosine'. Must be a positive integer.
    :type warmup_cosine_lr_warmup_epochs: int
    :param weight_decay: Value of weight decay when optimizer is 'sgd', 'adam', or 'adamw'. Must be
     a float in the range[0, 1].
    :type weight_decay: float
    """

    def __init__(
        self,
        *,
        advanced_settings: Optional[str] = None,
        ams_gradient: Optional[bool] = None,
        beta1: Optional[float] = None,
        beta2: Optional[float] = None,
        checkpoint_frequency: Optional[int] = None,
        checkpoint_run_id: Optional[str] = None,
        distributed: Optional[bool] = None,
        early_stopping: Optional[bool] = None,
        early_stopping_delay: Optional[int] = None,
        early_stopping_patience: Optional[int] = None,
        enable_onnx_normalization: Optional[bool] = None,
        evaluation_frequency: Optional[int] = None,
        gradient_accumulation_step: Optional[int] = None,
        layers_to_freeze: Optional[int] = None,
        learning_rate: Optional[float] = None,
        learning_rate_scheduler: Optional[LearningRateScheduler] = None,
        model_name: Optional[str] = None,
        momentum: Optional[float] = None,
        nesterov: Optional[bool] = None,
        number_of_epochs: Optional[int] = None,
        number_of_workers: Optional[int] = None,
        optimizer: Optional[StochasticOptimizer] = None,
        random_seed: Optional[int] = None,
        step_lr_gamma: Optional[float] = None,
        step_lr_step_size: Optional[int] = None,
        training_batch_size: Optional[int] = None,
        validation_batch_size: Optional[int] = None,
        warmup_cosine_lr_cycles: Optional[float] = None,
        warmup_cosine_lr_warmup_epochs: Optional[int] = None,
        weight_decay: Optional[float] = None,
    ):
        self.advanced_settings = advanced_settings
        self.ams_gradient = ams_gradient
        self.beta1 = beta1
        self.beta2 = beta2
        self.checkpoint_frequency = checkpoint_frequency
        self.checkpoint_run_id = checkpoint_run_id
        self.distributed = distributed
        self.early_stopping = early_stopping
        self.early_stopping_delay = early_stopping_delay
        self.early_stopping_patience = early_stopping_patience
        self.enable_onnx_normalization = enable_onnx_normalization
        self.evaluation_frequency = evaluation_frequency
        self.gradient_accumulation_step = gradient_accumulation_step
        self.layers_to_freeze = layers_to_freeze
        self.learning_rate = learning_rate
        self.learning_rate_scheduler = learning_rate_scheduler
        self.model_name = model_name
        self.momentum = momentum
        self.nesterov = nesterov
        self.number_of_epochs = number_of_epochs
        self.number_of_workers = number_of_workers
        self.optimizer = optimizer
        self.random_seed = random_seed
        self.step_lr_gamma = step_lr_gamma
        self.step_lr_step_size = step_lr_step_size
        self.training_batch_size = training_batch_size
        self.validation_batch_size = validation_batch_size
        self.warmup_cosine_lr_cycles = warmup_cosine_lr_cycles
        self.warmup_cosine_lr_warmup_epochs = warmup_cosine_lr_warmup_epochs
        self.weight_decay = weight_decay

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ImageModelDistributionSettings):
            return NotImplemented

        return (
            self.advanced_settings == other.advanced_settings
            and self.ams_gradient == other.ams_gradient
            and self.beta1 == other.beta1
            and self.beta2 == other.beta2
            and self.checkpoint_frequency == other.checkpoint_frequency
            and self.checkpoint_run_id == other.checkpoint_run_id
            and self.distributed == other.distributed
            and self.early_stopping == other.early_stopping
            and self.early_stopping_delay == other.early_stopping_delay
            and self.early_stopping_patience == other.early_stopping_patience
            and self.enable_onnx_normalization == other.enable_onnx_normalization
            and self.evaluation_frequency == other.evaluation_frequency
            and self.gradient_accumulation_step == other.gradient_accumulation_step
            and self.layers_to_freeze == other.layers_to_freeze
            and self.learning_rate == other.learning_rate
            and self.learning_rate_scheduler == other.learning_rate_scheduler
            and self.model_name == other.model_name
            and self.momentum == other.momentum
            and self.nesterov == other.nesterov
            and self.number_of_epochs == other.number_of_epochs
            and self.number_of_workers == other.number_of_workers
            and self.optimizer == other.optimizer
            and self.random_seed == other.random_seed
            and self.step_lr_gamma == other.step_lr_gamma
            and self.step_lr_step_size == other.step_lr_step_size
            and self.training_batch_size == other.training_batch_size
            and self.validation_batch_size == other.validation_batch_size
            and self.warmup_cosine_lr_cycles == other.warmup_cosine_lr_cycles
            and self.warmup_cosine_lr_warmup_epochs == other.warmup_cosine_lr_warmup_epochs
            and self.weight_decay == other.weight_decay
        )


class ImageModelSettingsClassification(ImageModelDistributionSettings):
    """Model settings for AutoML Image Classification tasks.

    :param advanced_settings: Settings for advanced scenarios.
    :type advanced_settings: str
    :param ams_gradient: Enable AMSGrad when optimizer is 'adam' or 'adamw'.
    :type ams_gradient: bool
    :param beta1: Value of 'beta1' when optimizer is 'adam' or 'adamw'. Must be a float in the range
     [0, 1].
    :type beta1: float
    :param beta2: Value of 'beta2' when optimizer is 'adam' or 'adamw'. Must be a float in the range
     [0, 1].
    :type beta2: float
    :param checkpoint_frequency: Frequency to store model checkpoints. Must be a positive integer.
    :type checkpoint_frequency: int
    :param checkpoint_run_id: The id of a previous run that has a pretrained checkpoint for
     incremental training.
    :type checkpoint_run_id: str
    :param distributed: Whether to use distributed training.
    :type distributed: bool
    :param early_stopping: Enable early stopping logic during training.
    :type early_stopping: bool
    :param early_stopping_delay: Minimum number of epochs or validation evaluations to wait before
     primary metric improvement
     is tracked for early stopping. Must be a positive integer.
    :type early_stopping_delay: int
    :param early_stopping_patience: Minimum number of epochs or validation evaluations with no
     primary metric improvement before
     the run is stopped. Must be a positive integer.
    :type early_stopping_patience: int
    :param enable_onnx_normalization: Enable normalization when exporting ONNX model.
    :type enable_onnx_normalization: bool
    :param evaluation_frequency: Frequency to evaluate validation dataset to get metric scores. Must
     be a positive integer.
    :type evaluation_frequency: int
    :param gradient_accumulation_step: Gradient accumulation means running a configured number of
     "GradAccumulationStep" steps without
     updating the model weights while accumulating the gradients of those steps, and then using
     the accumulated gradients to compute the weight updates. Must be a positive integer.
    :type gradient_accumulation_step: int
    :param layers_to_freeze: Number of layers to freeze for the model. Must be a positive integer.
     For instance, passing 2 as value for 'seresnext' means
     freezing layer0 and layer1. For a full list of models supported and details on layer freeze,
     please
     see: https://learn.microsoft.com/azure/machine-learning/how-to-auto-train-image-models.
    :type layers_to_freeze: int
    :param learning_rate: Initial learning rate. Must be a float in the range [0, 1].
    :type learning_rate: float
    :param learning_rate_scheduler: Type of learning rate scheduler. Must be 'warmup_cosine' or
     'step'. Possible values include: "None", "WarmupCosine", "Step".
    :type learning_rate_scheduler: str or
     ~azure.mgmt.machinelearningservices.models.LearningRateScheduler
    :param model_name: Name of the model to use for training.
     For more information on the available models please visit the official documentation:
     https://learn.microsoft.com/azure/machine-learning/how-to-auto-train-image-models.
    :type model_name: str
    :param momentum: Value of momentum when optimizer is 'sgd'. Must be a float in the range [0, 1].
    :type momentum: float
    :param nesterov: Enable nesterov when optimizer is 'sgd'.
    :type nesterov: bool
    :param number_of_epochs: Number of training epochs. Must be a positive integer.
    :type number_of_epochs: int
    :param number_of_workers: Number of data loader workers. Must be a non-negative integer.
    :type number_of_workers: int
    :param optimizer: Type of optimizer. Possible values include: "None", "Sgd", "Adam", "Adamw".
    :type optimizer: str or ~azure.mgmt.machinelearningservices.models.StochasticOptimizer
    :param random_seed: Random seed to be used when using deterministic training.
    :type random_seed: int
    :param step_lr_gamma: Value of gamma when learning rate scheduler is 'step'. Must be a float in
     the range [0, 1].
    :type step_lr_gamma: float
    :param step_lr_step_size: Value of step size when learning rate scheduler is 'step'. Must be a
     positive integer.
    :type step_lr_step_size: int
    :param training_batch_size: Training batch size. Must be a positive integer.
    :type training_batch_size: int
    :param validation_batch_size: Validation batch size. Must be a positive integer.
    :type validation_batch_size: int
    :param warmup_cosine_lr_cycles: Value of cosine cycle when learning rate scheduler is
     'warmup_cosine'. Must be a float in the range [0, 1].
    :type warmup_cosine_lr_cycles: float
    :param warmup_cosine_lr_warmup_epochs: Value of warmup epochs when learning rate scheduler is
     'warmup_cosine'. Must be a positive integer.
    :type warmup_cosine_lr_warmup_epochs: int
    :param weight_decay: Value of weight decay when optimizer is 'sgd', 'adam', or 'adamw'. Must be
     a float in the range[0, 1].
    :type weight_decay: float
    :param training_crop_size: Image crop size that is input to the neural network for the training
     dataset. Must be a positive integer.
    :type training_crop_size: int
    :param validation_crop_size: Image crop size that is input to the neural network for the
     validation dataset. Must be a positive integer.
    :type validation_crop_size: int
    :param validation_resize_size: Image size to which to resize before cropping for validation
     dataset. Must be a positive integer.
    :type validation_resize_size: int
    :param weighted_loss: Weighted loss. The accepted values are 0 for no weighted loss.
     1 for weighted loss with sqrt.(class_weights). 2 for weighted loss with class_weights. Must be
     0 or 1 or 2.
    :type weighted_loss: int

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_automl_image.py
            :start-after: [START automl.automl_image_job.image_classification_model_settings]
            :end-before: [END automl.automl_image_job.image_classification_model_settings]
            :language: python
            :dedent: 8
            :caption: Defining the automl image classification model settings.
    """

    def __init__(
        self,
        *,
        advanced_settings: Optional[str] = None,
        ams_gradient: Optional[bool] = None,
        beta1: Optional[float] = None,
        beta2: Optional[float] = None,
        checkpoint_frequency: Optional[int] = None,
        checkpoint_run_id: Optional[str] = None,
        distributed: Optional[bool] = None,
        early_stopping: Optional[bool] = None,
        early_stopping_delay: Optional[int] = None,
        early_stopping_patience: Optional[int] = None,
        enable_onnx_normalization: Optional[bool] = None,
        evaluation_frequency: Optional[int] = None,
        gradient_accumulation_step: Optional[int] = None,
        layers_to_freeze: Optional[int] = None,
        learning_rate: Optional[float] = None,
        learning_rate_scheduler: Optional[LearningRateScheduler] = None,
        model_name: Optional[str] = None,
        momentum: Optional[float] = None,
        nesterov: Optional[bool] = None,
        number_of_epochs: Optional[int] = None,
        number_of_workers: Optional[int] = None,
        optimizer: Optional[StochasticOptimizer] = None,
        random_seed: Optional[int] = None,
        step_lr_gamma: Optional[float] = None,
        step_lr_step_size: Optional[int] = None,
        training_batch_size: Optional[int] = None,
        validation_batch_size: Optional[int] = None,
        warmup_cosine_lr_cycles: Optional[float] = None,
        warmup_cosine_lr_warmup_epochs: Optional[int] = None,
        weight_decay: Optional[float] = None,
        training_crop_size: Optional[int] = None,
        validation_crop_size: Optional[int] = None,
        validation_resize_size: Optional[int] = None,
        weighted_loss: Optional[int] = None,
        **kwargs: Any,
    ):
        super(ImageModelSettingsClassification, self).__init__(
            advanced_settings=advanced_settings,
            ams_gradient=ams_gradient,
            beta1=beta1,
            beta2=beta2,
            checkpoint_frequency=checkpoint_frequency,
            checkpoint_run_id=checkpoint_run_id,
            distributed=distributed,
            early_stopping=early_stopping,
            early_stopping_delay=early_stopping_delay,
            early_stopping_patience=early_stopping_patience,
            enable_onnx_normalization=enable_onnx_normalization,
            evaluation_frequency=evaluation_frequency,
            gradient_accumulation_step=gradient_accumulation_step,
            layers_to_freeze=layers_to_freeze,
            learning_rate=learning_rate,
            learning_rate_scheduler=learning_rate_scheduler,
            model_name=model_name,
            momentum=momentum,
            nesterov=nesterov,
            number_of_epochs=number_of_epochs,
            number_of_workers=number_of_workers,
            optimizer=optimizer,
            random_seed=random_seed,
            step_lr_gamma=step_lr_gamma,
            step_lr_step_size=step_lr_step_size,
            training_batch_size=training_batch_size,
            validation_batch_size=validation_batch_size,
            warmup_cosine_lr_cycles=warmup_cosine_lr_cycles,
            warmup_cosine_lr_warmup_epochs=warmup_cosine_lr_warmup_epochs,
            weight_decay=weight_decay,
            **kwargs,
        )
        self.training_crop_size = training_crop_size
        self.validation_crop_size = validation_crop_size
        self.validation_resize_size = validation_resize_size
        self.weighted_loss = weighted_loss

    def _to_rest_object(self) -> RestImageModelSettingsClassification:
        return RestImageModelSettingsClassification(
            advanced_settings=self.advanced_settings,
            ams_gradient=self.ams_gradient,
            beta1=self.beta1,
            beta2=self.beta2,
            checkpoint_frequency=self.checkpoint_frequency,
            checkpoint_run_id=self.checkpoint_run_id,
            distributed=self.distributed,
            early_stopping=self.early_stopping,
            early_stopping_delay=self.early_stopping_delay,
            early_stopping_patience=self.early_stopping_patience,
            enable_onnx_normalization=self.enable_onnx_normalization,
            evaluation_frequency=self.evaluation_frequency,
            gradient_accumulation_step=self.gradient_accumulation_step,
            layers_to_freeze=self.layers_to_freeze,
            learning_rate=self.learning_rate,
            learning_rate_scheduler=self.learning_rate_scheduler,
            model_name=self.model_name,
            momentum=self.momentum,
            nesterov=self.nesterov,
            number_of_epochs=self.number_of_epochs,
            number_of_workers=self.number_of_workers,
            optimizer=self.optimizer,
            random_seed=self.random_seed,
            step_lr_gamma=self.step_lr_gamma,
            step_lr_step_size=self.step_lr_step_size,
            training_batch_size=self.training_batch_size,
            validation_batch_size=self.validation_batch_size,
            warmup_cosine_lr_cycles=self.warmup_cosine_lr_cycles,
            warmup_cosine_lr_warmup_epochs=self.warmup_cosine_lr_warmup_epochs,
            weight_decay=self.weight_decay,
            training_crop_size=self.training_crop_size,
            validation_crop_size=self.validation_crop_size,
            validation_resize_size=self.validation_resize_size,
            weighted_loss=self.weighted_loss,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestImageModelSettingsClassification) -> "ImageModelSettingsClassification":
        return cls(
            advanced_settings=obj.advanced_settings,
            ams_gradient=obj.ams_gradient,
            beta1=obj.beta1,
            beta2=obj.beta2,
            checkpoint_frequency=obj.checkpoint_frequency,
            checkpoint_run_id=obj.checkpoint_run_id,
            distributed=obj.distributed,
            early_stopping=obj.early_stopping,
            early_stopping_delay=obj.early_stopping_delay,
            early_stopping_patience=obj.early_stopping_patience,
            enable_onnx_normalization=obj.enable_onnx_normalization,
            evaluation_frequency=obj.evaluation_frequency,
            gradient_accumulation_step=obj.gradient_accumulation_step,
            layers_to_freeze=obj.layers_to_freeze,
            learning_rate=obj.learning_rate,
            learning_rate_scheduler=obj.learning_rate_scheduler,
            model_name=obj.model_name,
            momentum=obj.momentum,
            nesterov=obj.nesterov,
            number_of_epochs=obj.number_of_epochs,
            number_of_workers=obj.number_of_workers,
            optimizer=obj.optimizer,
            random_seed=obj.random_seed,
            step_lr_gamma=obj.step_lr_gamma,
            step_lr_step_size=obj.step_lr_step_size,
            training_batch_size=obj.training_batch_size,
            validation_batch_size=obj.validation_batch_size,
            warmup_cosine_lr_cycles=obj.warmup_cosine_lr_cycles,
            warmup_cosine_lr_warmup_epochs=obj.warmup_cosine_lr_warmup_epochs,
            weight_decay=obj.weight_decay,
            training_crop_size=obj.training_crop_size,
            validation_crop_size=obj.validation_crop_size,
            validation_resize_size=obj.validation_resize_size,
            weighted_loss=obj.weighted_loss,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ImageModelSettingsClassification):
            return NotImplemented

        return (
            super().__eq__(other)
            and self.training_crop_size == other.training_crop_size
            and self.validation_crop_size == other.validation_crop_size
            and self.validation_resize_size == other.validation_resize_size
            and self.weighted_loss == other.weighted_loss
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)


class ImageModelSettingsObjectDetection(ImageModelDistributionSettings):
    """Model settings for AutoML Image Object Detection Task.

    :param advanced_settings: Settings for advanced scenarios.
    :type advanced_settings: str
    :param ams_gradient: Enable AMSGrad when optimizer is 'adam' or 'adamw'.
    :type ams_gradient: bool
    :param beta1: Value of 'beta1' when optimizer is 'adam' or 'adamw'. Must be a float in the range
     [0, 1].
    :type beta1: float
    :param beta2: Value of 'beta2' when optimizer is 'adam' or 'adamw'. Must be a float in the range
     [0, 1].
    :type beta2: float
    :param checkpoint_frequency: Frequency to store model checkpoints. Must be a positive integer.
    :type checkpoint_frequency: int
    :param checkpoint_run_id: The id of a previous run that has a pretrained checkpoint for
     incremental training.
    :type checkpoint_run_id: str
    :param distributed: Whether to use distributed training.
    :type distributed: bool
    :param early_stopping: Enable early stopping logic during training.
    :type early_stopping: bool
    :param early_stopping_delay: Minimum number of epochs or validation evaluations to wait before
     primary metric improvement
     is tracked for early stopping. Must be a positive integer.
    :type early_stopping_delay: int
    :param early_stopping_patience: Minimum number of epochs or validation evaluations with no
     primary metric improvement before
     the run is stopped. Must be a positive integer.
    :type early_stopping_patience: int
    :param enable_onnx_normalization: Enable normalization when exporting ONNX model.
    :type enable_onnx_normalization: bool
    :param evaluation_frequency: Frequency to evaluate validation dataset to get metric scores. Must
     be a positive integer.
    :type evaluation_frequency: int
    :param gradient_accumulation_step: Gradient accumulation means running a configured number of
     "GradAccumulationStep" steps without
     updating the model weights while accumulating the gradients of those steps, and then using
     the accumulated gradients to compute the weight updates. Must be a positive integer.
    :type gradient_accumulation_step: int
    :param layers_to_freeze: Number of layers to freeze for the model. Must be a positive integer.
     For instance, passing 2 as value for 'seresnext' means
     freezing layer0 and layer1. For a full list of models supported and details on layer freeze,
     please
     see: https://learn.microsoft.com/azure/machine-learning/how-to-auto-train-image-models.
    :type layers_to_freeze: int
    :param learning_rate: Initial learning rate. Must be a float in the range [0, 1].
    :type learning_rate: float
    :param learning_rate_scheduler: Type of learning rate scheduler. Must be 'warmup_cosine' or
     'step'. Possible values include: "None", "WarmupCosine", "Step".
    :type learning_rate_scheduler: str or
     ~azure.mgmt.machinelearningservices.models.LearningRateScheduler
    :param model_name: Name of the model to use for training.
     For more information on the available models please visit the official documentation:
     https://learn.microsoft.com/azure/machine-learning/how-to-auto-train-image-models.
    :type model_name: str
    :param momentum: Value of momentum when optimizer is 'sgd'. Must be a float in the range [0, 1].
    :type momentum: float
    :param nesterov: Enable nesterov when optimizer is 'sgd'.
    :type nesterov: bool
    :param number_of_epochs: Number of training epochs. Must be a positive integer.
    :type number_of_epochs: int
    :param number_of_workers: Number of data loader workers. Must be a non-negative integer.
    :type number_of_workers: int
    :param optimizer: Type of optimizer. Possible values include: "None", "Sgd", "Adam", "Adamw".
    :type optimizer: str or ~azure.mgmt.machinelearningservices.models.StochasticOptimizer
    :param random_seed: Random seed to be used when using deterministic training.
    :type random_seed: int
    :param step_lr_gamma: Value of gamma when learning rate scheduler is 'step'. Must be a float in
     the range [0, 1].
    :type step_lr_gamma: float
    :param step_lr_step_size: Value of step size when learning rate scheduler is 'step'. Must be a
     positive integer.
    :type step_lr_step_size: int
    :param training_batch_size: Training batch size. Must be a positive integer.
    :type training_batch_size: int
    :param validation_batch_size: Validation batch size. Must be a positive integer.
    :type validation_batch_size: int
    :param warmup_cosine_lr_cycles: Value of cosine cycle when learning rate scheduler is
     'warmup_cosine'. Must be a float in the range [0, 1].
    :type warmup_cosine_lr_cycles: float
    :param warmup_cosine_lr_warmup_epochs: Value of warmup epochs when learning rate scheduler is
     'warmup_cosine'. Must be a positive integer.
    :type warmup_cosine_lr_warmup_epochs: int
    :param weight_decay: Value of weight decay when optimizer is 'sgd', 'adam', or 'adamw'. Must be
     a float in the range[0, 1].
    :type weight_decay: float
    :param box_detections_per_image: Maximum number of detections per image, for all classes. Must
     be a positive integer.
     Note: This settings is not supported for the 'yolov5' algorithm.
    :type box_detections_per_image: int
    :param box_score_threshold: During inference, only return proposals with a classification score
     greater than
     BoxScoreThreshold. Must be a float in the range[0, 1].
    :type box_score_threshold: float
    :param image_size: Image size for train and validation. Must be a positive integer.
     Note: The training run may get into CUDA OOM if the size is too big.
     Note: This settings is only supported for the 'yolov5' algorithm.
    :type image_size: int
    :param max_size: Maximum size of the image to be rescaled before feeding it to the backbone.
     Must be a positive integer. Note: training run may get into CUDA OOM if the size is too big.
     Note: This settings is not supported for the 'yolov5' algorithm.
    :type max_size: int
    :param min_size: Minimum size of the image to be rescaled before feeding it to the backbone.
     Must be a positive integer. Note: training run may get into CUDA OOM if the size is too big.
     Note: This settings is not supported for the 'yolov5' algorithm.
    :type min_size: int
    :param model_size: Model size. Must be 'small', 'medium', 'large'.
     Note: training run may get into CUDA OOM if the model size is too big.
     Note: This settings is only supported for the 'yolov5' algorithm. Possible values include:
     "None", "Small", "Medium", "Large", "ExtraLarge".
    :type model_size: str or ~azure.mgmt.machinelearningservices.models.ModelSize
    :param multi_scale: Enable multi-scale image by varying image size by +/- 50%.
     Note: training run may get into CUDA OOM if no sufficient GPU memory.
     Note: This settings is only supported for the 'yolov5' algorithm.
    :type multi_scale: bool
    :param nms_iou_threshold: IOU threshold used during inference in NMS post processing. Must be a
     float in the range [0, 1].
    :type nms_iou_threshold: float
    :param tile_grid_size: The grid size to use for tiling each image. Note: TileGridSize must not
     be
     None to enable small object detection logic. A string containing two integers in mxn format.
     Note: This settings is not supported for the 'yolov5' algorithm.
    :type tile_grid_size: str
    :param tile_overlap_ratio: Overlap ratio between adjacent tiles in each dimension. Must be float
     in the range [0, 1).
     Note: This settings is not supported for the 'yolov5' algorithm.
    :type tile_overlap_ratio: float
    :param tile_predictions_nms_threshold: The IOU threshold to use to perform NMS while merging
     predictions from tiles and image.
     Used in validation/ inference. Must be float in the range [0, 1].
     Note: This settings is not supported for the 'yolov5' algorithm.
    :type tile_predictions_nms_threshold: float
    :param validation_iou_threshold: IOU threshold to use when computing validation metric. Must be
     float in the range [0, 1].
    :type validation_iou_threshold: float
    :param validation_metric_type: Metric computation method to use for validation metrics. Possible
     values include: "None", "Coco", "Voc", "CocoVoc".
    :type validation_metric_type: str or
     ~azure.mgmt.machinelearningservices.models.ValidationMetricType
    :param log_training_metrics: indicates whether or not to log training metrics
    :type log_training_metrics: str or
     ~azure.mgmt.machinelearningservices.models.LogTrainingMetrics
    :param log_validation_loss: indicates whether or not to log validation loss
    :type log_validation_loss: str or
     ~azure.mgmt.machinelearningservices.models.LogValidationLoss

    .. literalinclude:: ../samples/ml_samples_automl_image.py
            :start-after: [START automl.automl_image_job.image_object_detection_model_settings]
            :end-before: [END automl.automl_image_job.image_object_detection_model_settings]
            :language: python
            :dedent: 8
            :caption: Defining the automl image object detection or instance segmentation model settings.
    """

    def __init__(
        self,
        *,
        advanced_settings: Optional[str] = None,
        ams_gradient: Optional[bool] = None,
        beta1: Optional[float] = None,
        beta2: Optional[float] = None,
        checkpoint_frequency: Optional[int] = None,
        checkpoint_run_id: Optional[str] = None,
        distributed: Optional[bool] = None,
        early_stopping: Optional[bool] = None,
        early_stopping_delay: Optional[int] = None,
        early_stopping_patience: Optional[int] = None,
        enable_onnx_normalization: Optional[bool] = None,
        evaluation_frequency: Optional[int] = None,
        gradient_accumulation_step: Optional[int] = None,
        layers_to_freeze: Optional[int] = None,
        learning_rate: Optional[float] = None,
        learning_rate_scheduler: Optional[LearningRateScheduler] = None,
        model_name: Optional[str] = None,
        momentum: Optional[float] = None,
        nesterov: Optional[bool] = None,
        number_of_epochs: Optional[int] = None,
        number_of_workers: Optional[int] = None,
        optimizer: Optional[StochasticOptimizer] = None,
        random_seed: Optional[int] = None,
        step_lr_gamma: Optional[float] = None,
        step_lr_step_size: Optional[int] = None,
        training_batch_size: Optional[int] = None,
        validation_batch_size: Optional[int] = None,
        warmup_cosine_lr_cycles: Optional[float] = None,
        warmup_cosine_lr_warmup_epochs: Optional[int] = None,
        weight_decay: Optional[float] = None,
        box_detections_per_image: Optional[int] = None,
        box_score_threshold: Optional[float] = None,
        image_size: Optional[int] = None,
        max_size: Optional[int] = None,
        min_size: Optional[int] = None,
        model_size: Optional[ModelSize] = None,
        multi_scale: Optional[bool] = None,
        nms_iou_threshold: Optional[float] = None,
        tile_grid_size: Optional[str] = None,
        tile_overlap_ratio: Optional[float] = None,
        tile_predictions_nms_threshold: Optional[float] = None,
        validation_iou_threshold: Optional[float] = None,
        validation_metric_type: Optional[ValidationMetricType] = None,
        log_training_metrics: Optional[LogTrainingMetrics] = None,
        log_validation_loss: Optional[LogValidationLoss] = None,
        **kwargs: Any,
    ):
        super(ImageModelSettingsObjectDetection, self).__init__(
            advanced_settings=advanced_settings,
            ams_gradient=ams_gradient,
            beta1=beta1,
            beta2=beta2,
            checkpoint_frequency=checkpoint_frequency,
            checkpoint_run_id=checkpoint_run_id,
            distributed=distributed,
            early_stopping=early_stopping,
            early_stopping_delay=early_stopping_delay,
            early_stopping_patience=early_stopping_patience,
            enable_onnx_normalization=enable_onnx_normalization,
            evaluation_frequency=evaluation_frequency,
            gradient_accumulation_step=gradient_accumulation_step,
            layers_to_freeze=layers_to_freeze,
            learning_rate=learning_rate,
            learning_rate_scheduler=learning_rate_scheduler,
            model_name=model_name,
            momentum=momentum,
            nesterov=nesterov,
            number_of_epochs=number_of_epochs,
            number_of_workers=number_of_workers,
            optimizer=optimizer,
            random_seed=random_seed,
            step_lr_gamma=step_lr_gamma,
            step_lr_step_size=step_lr_step_size,
            training_batch_size=training_batch_size,
            validation_batch_size=validation_batch_size,
            warmup_cosine_lr_cycles=warmup_cosine_lr_cycles,
            warmup_cosine_lr_warmup_epochs=warmup_cosine_lr_warmup_epochs,
            weight_decay=weight_decay,
            **kwargs,
        )
        self.box_detections_per_image = box_detections_per_image
        self.box_score_threshold = box_score_threshold
        self.image_size = image_size
        self.max_size = max_size
        self.min_size = min_size
        self.model_size = model_size
        self.multi_scale = multi_scale
        self.nms_iou_threshold = nms_iou_threshold
        self.tile_grid_size = tile_grid_size
        self.tile_overlap_ratio = tile_overlap_ratio
        self.tile_predictions_nms_threshold = tile_predictions_nms_threshold
        self.validation_iou_threshold = validation_iou_threshold
        self.validation_metric_type = validation_metric_type
        self.log_training_metrics = log_training_metrics
        self.log_validation_loss = log_validation_loss

    def _to_rest_object(self) -> RestImageModelSettingsObjectDetection:
        return RestImageModelSettingsObjectDetection(
            advanced_settings=self.advanced_settings,
            ams_gradient=self.ams_gradient,
            beta1=self.beta1,
            beta2=self.beta2,
            checkpoint_frequency=self.checkpoint_frequency,
            checkpoint_run_id=self.checkpoint_run_id,
            distributed=self.distributed,
            early_stopping=self.early_stopping,
            early_stopping_delay=self.early_stopping_delay,
            early_stopping_patience=self.early_stopping_patience,
            enable_onnx_normalization=self.enable_onnx_normalization,
            evaluation_frequency=self.evaluation_frequency,
            gradient_accumulation_step=self.gradient_accumulation_step,
            layers_to_freeze=self.layers_to_freeze,
            learning_rate=self.learning_rate,
            learning_rate_scheduler=self.learning_rate_scheduler,
            model_name=self.model_name,
            momentum=self.momentum,
            nesterov=self.nesterov,
            number_of_epochs=self.number_of_epochs,
            number_of_workers=self.number_of_workers,
            optimizer=self.optimizer,
            random_seed=self.random_seed,
            step_lr_gamma=self.step_lr_gamma,
            step_lr_step_size=self.step_lr_step_size,
            training_batch_size=self.training_batch_size,
            validation_batch_size=self.validation_batch_size,
            warmup_cosine_lr_cycles=self.warmup_cosine_lr_cycles,
            warmup_cosine_lr_warmup_epochs=self.warmup_cosine_lr_warmup_epochs,
            weight_decay=self.weight_decay,
            box_detections_per_image=self.box_detections_per_image,
            box_score_threshold=self.box_score_threshold,
            image_size=self.image_size,
            max_size=self.max_size,
            min_size=self.min_size,
            model_size=self.model_size,
            multi_scale=self.multi_scale,
            nms_iou_threshold=self.nms_iou_threshold,
            tile_grid_size=self.tile_grid_size,
            tile_overlap_ratio=self.tile_overlap_ratio,
            tile_predictions_nms_threshold=self.tile_predictions_nms_threshold,
            validation_iou_threshold=self.validation_iou_threshold,
            validation_metric_type=self.validation_metric_type,
            log_training_metrics=self.log_training_metrics,
            log_validation_loss=self.log_validation_loss,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestImageModelSettingsObjectDetection) -> "ImageModelSettingsObjectDetection":
        return cls(
            advanced_settings=obj.advanced_settings,
            ams_gradient=obj.ams_gradient,
            beta1=obj.beta1,
            beta2=obj.beta2,
            checkpoint_frequency=obj.checkpoint_frequency,
            checkpoint_run_id=obj.checkpoint_run_id,
            distributed=obj.distributed,
            early_stopping=obj.early_stopping,
            early_stopping_delay=obj.early_stopping_delay,
            early_stopping_patience=obj.early_stopping_patience,
            enable_onnx_normalization=obj.enable_onnx_normalization,
            evaluation_frequency=obj.evaluation_frequency,
            gradient_accumulation_step=obj.gradient_accumulation_step,
            layers_to_freeze=obj.layers_to_freeze,
            learning_rate=obj.learning_rate,
            learning_rate_scheduler=obj.learning_rate_scheduler,
            model_name=obj.model_name,
            momentum=obj.momentum,
            nesterov=obj.nesterov,
            number_of_epochs=obj.number_of_epochs,
            number_of_workers=obj.number_of_workers,
            optimizer=obj.optimizer,
            random_seed=obj.random_seed,
            step_lr_gamma=obj.step_lr_gamma,
            step_lr_step_size=obj.step_lr_step_size,
            training_batch_size=obj.training_batch_size,
            validation_batch_size=obj.validation_batch_size,
            warmup_cosine_lr_cycles=obj.warmup_cosine_lr_cycles,
            warmup_cosine_lr_warmup_epochs=obj.warmup_cosine_lr_warmup_epochs,
            weight_decay=obj.weight_decay,
            box_detections_per_image=obj.box_detections_per_image,
            box_score_threshold=obj.box_score_threshold,
            image_size=obj.image_size,
            max_size=obj.max_size,
            min_size=obj.min_size,
            model_size=obj.model_size,
            multi_scale=obj.multi_scale,
            nms_iou_threshold=obj.nms_iou_threshold,
            tile_grid_size=obj.tile_grid_size,
            tile_overlap_ratio=obj.tile_overlap_ratio,
            tile_predictions_nms_threshold=obj.tile_predictions_nms_threshold,
            validation_iou_threshold=obj.validation_iou_threshold,
            validation_metric_type=obj.validation_metric_type,
            log_training_metrics=obj.log_training_metrics,
            log_validation_loss=obj.log_validation_loss,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ImageModelSettingsObjectDetection):
            return NotImplemented

        return (
            super().__eq__(other)
            and self.box_detections_per_image == other.box_detections_per_image
            and self.box_score_threshold == other.box_score_threshold
            and self.image_size == other.image_size
            and self.max_size == other.max_size
            and self.min_size == other.min_size
            and self.model_size == other.model_size
            and self.multi_scale == other.multi_scale
            and self.nms_iou_threshold == other.nms_iou_threshold
            and self.tile_grid_size == other.tile_grid_size
            and self.tile_overlap_ratio == other.tile_overlap_ratio
            and self.tile_predictions_nms_threshold == other.tile_predictions_nms_threshold
            and self.validation_iou_threshold == other.validation_iou_threshold
            and self.validation_metric_type == other.validation_metric_type
            and self.log_training_metrics == other.log_training_metrics
            and self.log_validation_loss == other.log_validation_loss
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import urllib.parse
from io import IOBase
from typing import IO, Any, Callable, Dict, Iterable, Optional, TypeVar, Union, cast, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.polling import LROPoller, NoPolling, PollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.arm_polling import ARMPolling

from .. import models as _models
from .._serialization import Serializer
from .._vendor import _convert_request

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_request(
    resource_group_name: str,
    workspace_name: str,
    subscription_id: str,
    *,
    name: Optional[str] = None,
    count: Optional[int] = None,
    compute_type: Optional[Union[str, _models.EndpointComputeType]] = None,
    skip: Optional[str] = None,
    tags: Optional[str] = None,
    properties: Optional[str] = None,
    order_by: Optional[Union[str, _models.OrderString]] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if name is not None:
        _params["name"] = _SERIALIZER.query("name", name, "str")
    if count is not None:
        _params["count"] = _SERIALIZER.query("count", count, "int")
    if compute_type is not None:
        _params["computeType"] = _SERIALIZER.query("compute_type", compute_type, "str")
    if skip is not None:
        _params["$skip"] = _SERIALIZER.query("skip", skip, "str")
    if tags is not None:
        _params["tags"] = _SERIALIZER.query("tags", tags, "str")
    if properties is not None:
        _params["properties"] = _SERIALIZER.query("properties", properties, "str")
    if order_by is not None:
        _params["orderBy"] = _SERIALIZER.query("order_by", order_by, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_delete_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_update_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PATCH", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_or_update_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url(
            "endpoint_name", endpoint_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9\-_]{0,254}$"
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_keys_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/listKeys",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_regenerate_keys_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/regenerateKeys",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_token_request(
    resource_group_name: str, workspace_name: str, endpoint_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-10-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/token",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "workspaceName": _SERIALIZER.url(
            "workspace_name", workspace_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_-]{2,32}$"
        ),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


class OnlineEndpointsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.machinelearningservices.AzureMachineLearningServices`'s
        :attr:`online_endpoints` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: Optional[str] = None,
        count: Optional[int] = None,
        compute_type: Optional[Union[str, _models.EndpointComputeType]] = None,
        skip: Optional[str] = None,
        tags: Optional[str] = None,
        properties: Optional[str] = None,
        order_by: Optional[Union[str, _models.OrderString]] = None,
        **kwargs: Any
    ) -> Iterable["_models.OnlineEndpoint"]:
        """List Online Endpoints.

        List Online Endpoints.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param name: Name of the endpoint. Default value is None.
        :type name: str
        :param count: Number of endpoints to be retrieved in a page of results. Default value is None.
        :type count: int
        :param compute_type: EndpointComputeType to be filtered by. Known values are: "Managed",
         "Kubernetes", and "AzureMLCompute". Default value is None.
        :type compute_type: str or ~azure.mgmt.machinelearningservices.models.EndpointComputeType
        :param skip: Continuation token for pagination. Default value is None.
        :type skip: str
        :param tags: A set of tags with which to filter the returned models. It is a comma separated
         string of tags key or tags key=value. Example: tagKey1,tagKey2,tagKey3=value3 . Default value
         is None.
        :type tags: str
        :param properties: A set of properties with which to filter the returned models. It is a comma
         separated string of properties key and/or properties key=value Example:
         propKey1,propKey2,propKey3=value3 . Default value is None.
        :type properties: str
        :param order_by: The option to order the response. Known values are: "CreatedAtDesc",
         "CreatedAtAsc", "UpdatedAtDesc", and "UpdatedAtAsc". Default value is None.
        :type order_by: str or ~azure.mgmt.machinelearningservices.models.OrderString
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either OnlineEndpoint or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.OnlineEndpointTrackedResourceArmPaginatedResult] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_request(
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    subscription_id=self._config.subscription_id,
                    name=name,
                    count=count,
                    compute_type=compute_type,
                    skip=skip,
                    tags=tags,
                    properties=properties,
                    order_by=order_by,
                    api_version=api_version,
                    template_url=self.list.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("OnlineEndpointTrackedResourceArmPaginatedResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    list.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints"
    }

    def _delete_initial(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, workspace_name: str, endpoint_name: str, **kwargs: Any
    ) -> None:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_delete_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self._delete_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["x-ms-async-operation-timeout"] = self._deserialize(
                "duration", response.headers.get("x-ms-async-operation-timeout")
            )
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))
            response_headers["Retry-After"] = self._deserialize("int", response.headers.get("Retry-After"))

        if cls:
            return cls(pipeline_response, None, response_headers)

    _delete_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    @distributed_trace
    def begin_delete(
        self, resource_group_name: str, workspace_name: str, endpoint_name: str, **kwargs: Any
    ) -> LROPoller[None]:
        """Delete Online Endpoint (asynchronous).

        Delete Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._delete_initial(  # type: ignore
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                endpoint_name=endpoint_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})

        if polling is True:
            polling_method: PollingMethod = cast(
                PollingMethod, ARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_delete.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    @distributed_trace
    def get(
        self, resource_group_name: str, workspace_name: str, endpoint_name: str, **kwargs: Any
    ) -> _models.OnlineEndpoint:
        """Get Online Endpoint.

        Get Online Endpoint.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: OnlineEndpoint or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.OnlineEndpoint
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.OnlineEndpoint] = kwargs.pop("cls", None)

        request = build_get_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("OnlineEndpoint", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    def _update_initial(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: Union[_models.PartialMinimalTrackedResourceWithIdentity, IO],
        **kwargs: Any
    ) -> Optional[_models.OnlineEndpoint]:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Optional[_models.OnlineEndpoint]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _json = self._serialize.body(body, "PartialMinimalTrackedResourceWithIdentity")

        request = build_update_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self._update_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = None
        response_headers = {}
        if response.status_code == 200:
            deserialized = self._deserialize("OnlineEndpoint", pipeline_response)

        if response.status_code == 202:
            response_headers["x-ms-async-operation-timeout"] = self._deserialize(
                "duration", response.headers.get("x-ms-async-operation-timeout")
            )
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))
            response_headers["Retry-After"] = self._deserialize("int", response.headers.get("Retry-After"))

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    _update_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    @overload
    def begin_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: _models.PartialMinimalTrackedResourceWithIdentity,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.OnlineEndpoint]:
        """Update Online Endpoint (asynchronous).

        Update Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: Online Endpoint entity to apply during operation. Required.
        :type body:
         ~azure.mgmt.machinelearningservices.models.PartialMinimalTrackedResourceWithIdentity
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either OnlineEndpoint or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.LROPoller[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.OnlineEndpoint]:
        """Update Online Endpoint (asynchronous).

        Update Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: Online Endpoint entity to apply during operation. Required.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either OnlineEndpoint or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.LROPoller[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: Union[_models.PartialMinimalTrackedResourceWithIdentity, IO],
        **kwargs: Any
    ) -> LROPoller[_models.OnlineEndpoint]:
        """Update Online Endpoint (asynchronous).

        Update Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: Online Endpoint entity to apply during operation. Is either a
         PartialMinimalTrackedResourceWithIdentity type or a IO type. Required.
        :type body:
         ~azure.mgmt.machinelearningservices.models.PartialMinimalTrackedResourceWithIdentity or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either OnlineEndpoint or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.LROPoller[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.OnlineEndpoint] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._update_initial(
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                endpoint_name=endpoint_name,
                body=body,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("OnlineEndpoint", pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_update.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    def _create_or_update_initial(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: Union[_models.OnlineEndpoint, IO],
        **kwargs: Any
    ) -> _models.OnlineEndpoint:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.OnlineEndpoint] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _json = self._serialize.body(body, "OnlineEndpoint")

        request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self._create_or_update_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 200:
            deserialized = self._deserialize("OnlineEndpoint", pipeline_response)

        if response.status_code == 201:
            response_headers["x-ms-async-operation-timeout"] = self._deserialize(
                "duration", response.headers.get("x-ms-async-operation-timeout")
            )
            response_headers["Azure-AsyncOperation"] = self._deserialize(
                "str", response.headers.get("Azure-AsyncOperation")
            )

            deserialized = self._deserialize("OnlineEndpoint", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    _create_or_update_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    @overload
    def begin_create_or_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: _models.OnlineEndpoint,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.OnlineEndpoint]:
        """Create or update Online Endpoint (asynchronous).

        Create or update Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: Online Endpoint entity to apply during operation. Required.
        :type body: ~azure.mgmt.machinelearningservices.models.OnlineEndpoint
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either OnlineEndpoint or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.LROPoller[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_create_or_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.OnlineEndpoint]:
        """Create or update Online Endpoint (asynchronous).

        Create or update Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: Online Endpoint entity to apply during operation. Required.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either OnlineEndpoint or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.LROPoller[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_create_or_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: Union[_models.OnlineEndpoint, IO],
        **kwargs: Any
    ) -> LROPoller[_models.OnlineEndpoint]:
        """Create or update Online Endpoint (asynchronous).

        Create or update Online Endpoint (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: Online Endpoint entity to apply during operation. Is either a OnlineEndpoint type
         or a IO type. Required.
        :type body: ~azure.mgmt.machinelearningservices.models.OnlineEndpoint or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either OnlineEndpoint or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.LROPoller[~azure.mgmt.machinelearningservices.models.OnlineEndpoint]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.OnlineEndpoint] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._create_or_update_initial(
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                endpoint_name=endpoint_name,
                body=body,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("OnlineEndpoint", pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(
                PollingMethod, ARMPolling(lro_delay, lro_options={"final-state-via": "original-uri"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_create_or_update.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}"
    }

    @distributed_trace
    def list_keys(
        self, resource_group_name: str, workspace_name: str, endpoint_name: str, **kwargs: Any
    ) -> _models.EndpointAuthKeys:
        """List EndpointAuthKeys for an Endpoint using Key-based authentication.

        List EndpointAuthKeys for an Endpoint using Key-based authentication.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EndpointAuthKeys or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.EndpointAuthKeys
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.EndpointAuthKeys] = kwargs.pop("cls", None)

        request = build_list_keys_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.list_keys.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("EndpointAuthKeys", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_keys.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/listKeys"
    }

    def _regenerate_keys_initial(  # pylint: disable=inconsistent-return-statements
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: Union[_models.RegenerateEndpointKeysRequest, IO],
        **kwargs: Any
    ) -> None:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _json = self._serialize.body(body, "RegenerateEndpointKeysRequest")

        request = build_regenerate_keys_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self._regenerate_keys_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))
            response_headers["Retry-After"] = self._deserialize("int", response.headers.get("Retry-After"))

        if cls:
            return cls(pipeline_response, None, response_headers)

    _regenerate_keys_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/regenerateKeys"
    }

    @overload
    def begin_regenerate_keys(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: _models.RegenerateEndpointKeysRequest,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[None]:
        """Regenerate EndpointAuthKeys for an Endpoint using Key-based authentication (asynchronous).

        Regenerate EndpointAuthKeys for an Endpoint using Key-based authentication (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: RegenerateKeys request . Required.
        :type body: ~azure.mgmt.machinelearningservices.models.RegenerateEndpointKeysRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_regenerate_keys(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[None]:
        """Regenerate EndpointAuthKeys for an Endpoint using Key-based authentication (asynchronous).

        Regenerate EndpointAuthKeys for an Endpoint using Key-based authentication (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: RegenerateKeys request . Required.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_regenerate_keys(
        self,
        resource_group_name: str,
        workspace_name: str,
        endpoint_name: str,
        body: Union[_models.RegenerateEndpointKeysRequest, IO],
        **kwargs: Any
    ) -> LROPoller[None]:
        """Regenerate EndpointAuthKeys for an Endpoint using Key-based authentication (asynchronous).

        Regenerate EndpointAuthKeys for an Endpoint using Key-based authentication (asynchronous).

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :param body: RegenerateKeys request . Is either a RegenerateEndpointKeysRequest type or a IO
         type. Required.
        :type body: ~azure.mgmt.machinelearningservices.models.RegenerateEndpointKeysRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._regenerate_keys_initial(  # type: ignore
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                endpoint_name=endpoint_name,
                body=body,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})

        if polling is True:
            polling_method: PollingMethod = cast(
                PollingMethod, ARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_regenerate_keys.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/regenerateKeys"
    }

    @distributed_trace
    def get_token(
        self, resource_group_name: str, workspace_name: str, endpoint_name: str, **kwargs: Any
    ) -> _models.EndpointAuthToken:
        """Retrieve a valid AML token for an Endpoint using AMLToken-based authentication.

        Retrieve a valid AML token for an Endpoint using AMLToken-based authentication.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param endpoint_name: Online Endpoint name. Required.
        :type endpoint_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EndpointAuthToken or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.EndpointAuthToken
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.EndpointAuthToken] = kwargs.pop("cls", None)

        request = build_get_token_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            endpoint_name=endpoint_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get_token.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("EndpointAuthToken", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_token.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/onlineEndpoints/{endpointName}/token"
    }



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: ml_samples_automl_nlp.py
DESCRIPTION:
    These samples demonstrate how to use AutoML NLP functions
USAGE:
    python ml_samples_automl_nlp.py

"""

import os


class AutoMLNLPSamples(object):
    def automl_nlp_jobs(self):
        # [START automl.text_classification]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        test_classification_job = automl.text_classification(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="Sentiment",
            primary_metric="accuracy",
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.text_classification]

        # [START automl.text_classification_multilabel]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        text_classification_multilabel_job = automl.text_classification_multilabel(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="terms",
            primary_metric="accuracy",
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.text_classification_multilabel]

        # [START automl.text_ner]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        text_ner_job = automl.text_ner(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.text_ner]

        # [START automl.automl_nlp_job.text_classification_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        text_classification_job = automl.TextClassificationJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="terms",
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.automl_nlp_job.text_classification_job]

        # [START automl.text_classification_multilabel_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        text_classification_multilabel_job = automl.TextClassificationMultilabelJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="terms",
            primary_metric="accuracy",
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.text_classification_multilabel_job]

        # [START automl.text_ner_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        text_ner_job = automl.TextNerJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.text_ner_job]

        # [START automl.nlp_sweep_settings]
        from azure.ai.ml import automl
        from azure.ai.ml.sweep import BanditPolicy

        nlp_sweep_settings = automl.NlpSweepSettings(
            sampling_algorithm="Grid",
            early_termination=BanditPolicy(evaluation_interval=2, slack_factor=0.05, delay_evaluation=6),
        )
        # [END automl.nlp_sweep_settings]

        # [START automl.nlp_search_space]
        from azure.ai.ml import automl
        from azure.ai.ml.constants import NlpLearningRateScheduler
        from azure.ai.ml.sweep import Uniform

        nlp_search_space = automl.NlpSearchSpace(
            learning_rate_scheduler=NlpLearningRateScheduler.LINEAR,
            warmup_ratio=0.1,
            model_name="roberta-base",
            weight_decay=Uniform(0.01, 0.1),
        )
        # [END automl.nlp_search_space]

        # [START automl.nlp_limit_settings]
        from azure.ai.ml import automl

        nlp_limit_settings = automl.NlpLimitSettings(
            max_concurrent_trials=2, max_trials=4, max_nodes=4, timeout_minutes=120
        )
        # [END automl.nlp_limit_settings]

        # [START automl.nlp_fixed_parameters]
        from azure.ai.ml import automl
        from azure.ai.ml.constants import NlpLearningRateScheduler

        nlp_fixed_parameters = automl.NlpFixedParameters(
            model_name="roberta-base",
            learning_rate_scheduler=NlpLearningRateScheduler.LINEAR,
            warmup_ratio=0.1,
        )
        # [END automl.nlp_fixed_parameters]

        # [START automl.nlp_featurization_settings]
        from azure.ai.ml import automl

        nlp_featurization_settings = automl.NlpFeaturizationSettings(dataset_language="eng")
        # [END automl.nlp_featurization_settings]


if __name__ == "__main__":
    sample = AutoMLNLPSamples()
    sample.automl_nlp_jobs()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AccountKeyDatastoreCredentials
    from ._models_py3 import AccountKeyDatastoreSecrets
    from ._models_py3 import AcrDetail
    from ._models_py3 import AmlToken
    from ._models_py3 import AssetBase
    from ._models_py3 import AssetContainer
    from ._models_py3 import AssetReferenceBase
    from ._models_py3 import AzureBlobDatastore
    from ._models_py3 import AzureDataLakeGen1Datastore
    from ._models_py3 import AzureDataLakeGen2Datastore
    from ._models_py3 import AzureFileDatastore
    from ._models_py3 import AzureMLBatchInferencingServer
    from ._models_py3 import AzureMLOnlineInferencingServer
    from ._models_py3 import BanditPolicy
    from ._models_py3 import BaseEnvironmentId
    from ._models_py3 import BaseEnvironmentSource
    from ._models_py3 import BasicBinding
    from ._models_py3 import Binding
    from ._models_py3 import BlobReferenceForConsumptionDto
    from ._models_py3 import BlobReferenceSASRequestDto
    from ._models_py3 import BlobReferenceSASResponseDto
    from ._models_py3 import BuildContext
    from ._models_py3 import CertificateDatastoreCredentials
    from ._models_py3 import CertificateDatastoreSecrets
    from ._models_py3 import CodeConfiguration
    from ._models_py3 import CodeContainerData
    from ._models_py3 import CodeContainerDetails
    from ._models_py3 import CodeContainerResourceArmPaginatedResult
    from ._models_py3 import CodeVersionData
    from ._models_py3 import CodeVersionDetails
    from ._models_py3 import CodeVersionResourceArmPaginatedResult
    from ._models_py3 import CommandJob
    from ._models_py3 import CommandJobLimits
    from ._models_py3 import ComponentContainerData
    from ._models_py3 import ComponentContainerDetails
    from ._models_py3 import ComponentContainerResourceArmPaginatedResult
    from ._models_py3 import ComponentJob
    from ._models_py3 import ComponentVersionData
    from ._models_py3 import ComponentVersionDetails
    from ._models_py3 import ComponentVersionResourceArmPaginatedResult
    from ._models_py3 import CustomInferencingServer
    from ._models_py3 import DataContainerData
    from ._models_py3 import DataContainerDetails
    from ._models_py3 import DataContainerResourceArmPaginatedResult
    from ._models_py3 import DataPathAssetReference
    from ._models_py3 import DataReferenceCredentialDto
    from ._models_py3 import DataVersionBaseData
    from ._models_py3 import DataVersionBaseDetails
    from ._models_py3 import DataVersionBaseResourceArmPaginatedResult
    from ._models_py3 import Datastore
    from ._models_py3 import DatastoreCredentials
    from ._models_py3 import DatastoreSecrets
    from ._models_py3 import DistributionConfiguration
    from ._models_py3 import DockerCredentialDto
    from ._models_py3 import EarlyTerminationPolicy
    from ._models_py3 import EnvironmentContainerData
    from ._models_py3 import EnvironmentContainerDetails
    from ._models_py3 import EnvironmentContainerResourceArmPaginatedResult
    from ._models_py3 import EnvironmentVersionData
    from ._models_py3 import EnvironmentVersionDetails
    from ._models_py3 import EnvironmentVersionResourceArmPaginatedResult
    from ._models_py3 import ErrorAdditionalInfo
    from ._models_py3 import ErrorDetail
    from ._models_py3 import ErrorResponse
    from ._models_py3 import FlavorData
    from ._models_py3 import IdAssetReference
    from ._models_py3 import IdentityConfiguration
    from ._models_py3 import ImageReferenceForConsumptionDto
    from ._models_py3 import InferenceContainerProperties
    from ._models_py3 import InferencingServer
    from ._models_py3 import IntellectualProperty
    from ._models_py3 import Job
    from ._models_py3 import JobBase
    from ._models_py3 import JobInput
    from ._models_py3 import JobInputDataset
    from ._models_py3 import JobInputLiteral
    from ._models_py3 import JobInputUri
    from ._models_py3 import JobLimits
    from ._models_py3 import JobOutput
    from ._models_py3 import JobOutputDataset
    from ._models_py3 import JobOutputUri
    from ._models_py3 import JobService
    from ._models_py3 import MLTableData
    from ._models_py3 import ManagedIdentity
    from ._models_py3 import ManagedIdentityCredentialDto
    from ._models_py3 import MedianStoppingPolicy
    from ._models_py3 import ModelConfiguration
    from ._models_py3 import ModelContainerData
    from ._models_py3 import ModelContainerDetails
    from ._models_py3 import ModelContainerResourceArmPaginatedResult
    from ._models_py3 import ModelPackageInput
    from ._models_py3 import ModelVersionData
    from ._models_py3 import ModelVersionDetails
    from ._models_py3 import ModelVersionResourceArmPaginatedResult
    from ._models_py3 import Mpi
    from ._models_py3 import NoneDatastoreCredentials
    from ._models_py3 import Objective
    from ._models_py3 import OnlineInferenceConfiguration
    from ._models_py3 import OutputPathAssetReference
    from ._models_py3 import PackageInputPathBase
    from ._models_py3 import PackageInputPathId
    from ._models_py3 import PackageInputPathUrl
    from ._models_py3 import PackageInputPathVersion
    from ._models_py3 import PackageRequest
    from ._models_py3 import PackageResponse
    from ._models_py3 import PipelineJob
    from ._models_py3 import PyTorch
    from ._models_py3 import Resource
    from ._models_py3 import ResourceBase
    from ._models_py3 import ResourceConfiguration
    from ._models_py3 import ResourceManagementAssetReferenceData
    from ._models_py3 import ResourceManagementAssetReferenceDetails
    from ._models_py3 import Route
    from ._models_py3 import SASCredentialDto
    from ._models_py3 import SasDatastoreCredentials
    from ._models_py3 import SasDatastoreSecrets
    from ._models_py3 import ServicePrincipalDatastoreCredentials
    from ._models_py3 import ServicePrincipalDatastoreSecrets
    from ._models_py3 import SweepJob
    from ._models_py3 import SweepJobLimits
    from ._models_py3 import SystemData
    from ._models_py3 import TemporaryDataReferenceRequestDto
    from ._models_py3 import TemporaryDataReferenceResponseDto
    from ._models_py3 import TensorFlow
    from ._models_py3 import TrialComponent
    from ._models_py3 import TritonInferencingServer
    from ._models_py3 import TruncationSelectionPolicy
    from ._models_py3 import UriFileDataVersion
    from ._models_py3 import UriFolderDataVersion
    from ._models_py3 import UriReference
except (SyntaxError, ImportError):
    from ._models import AccountKeyDatastoreCredentials  # type: ignore
    from ._models import AccountKeyDatastoreSecrets  # type: ignore
    from ._models import AcrDetail  # type: ignore
    from ._models import AmlToken  # type: ignore
    from ._models import AssetBase  # type: ignore
    from ._models import AssetContainer  # type: ignore
    from ._models import AssetReferenceBase  # type: ignore
    from ._models import AzureBlobDatastore  # type: ignore
    from ._models import AzureDataLakeGen1Datastore  # type: ignore
    from ._models import AzureDataLakeGen2Datastore  # type: ignore
    from ._models import AzureFileDatastore  # type: ignore
    from ._models import AzureMLBatchInferencingServer  # type: ignore
    from ._models import AzureMLOnlineInferencingServer  # type: ignore
    from ._models import BanditPolicy  # type: ignore
    from ._models import BaseEnvironmentId  # type: ignore
    from ._models import BaseEnvironmentSource  # type: ignore
    from ._models import BasicBinding  # type: ignore
    from ._models import Binding  # type: ignore
    from ._models import BlobReferenceForConsumptionDto  # type: ignore
    from ._models import BlobReferenceSASRequestDto  # type: ignore
    from ._models import BlobReferenceSASResponseDto  # type: ignore
    from ._models import BuildContext  # type: ignore
    from ._models import CertificateDatastoreCredentials  # type: ignore
    from ._models import CertificateDatastoreSecrets  # type: ignore
    from ._models import CodeConfiguration  # type: ignore
    from ._models import CodeContainerData  # type: ignore
    from ._models import CodeContainerDetails  # type: ignore
    from ._models import CodeContainerResourceArmPaginatedResult  # type: ignore
    from ._models import CodeVersionData  # type: ignore
    from ._models import CodeVersionDetails  # type: ignore
    from ._models import CodeVersionResourceArmPaginatedResult  # type: ignore
    from ._models import CommandJob  # type: ignore
    from ._models import CommandJobLimits  # type: ignore
    from ._models import ComponentContainerData  # type: ignore
    from ._models import ComponentContainerDetails  # type: ignore
    from ._models import ComponentContainerResourceArmPaginatedResult  # type: ignore
    from ._models import ComponentJob  # type: ignore
    from ._models import ComponentVersionData  # type: ignore
    from ._models import ComponentVersionDetails  # type: ignore
    from ._models import ComponentVersionResourceArmPaginatedResult  # type: ignore
    from ._models import CustomInferencingServer  # type: ignore
    from ._models import DataContainerData  # type: ignore
    from ._models import DataContainerDetails  # type: ignore
    from ._models import DataContainerResourceArmPaginatedResult  # type: ignore
    from ._models import DataPathAssetReference  # type: ignore
    from ._models import DataReferenceCredentialDto  # type: ignore
    from ._models import DataVersionBaseData  # type: ignore
    from ._models import DataVersionBaseDetails  # type: ignore
    from ._models import DataVersionBaseResourceArmPaginatedResult  # type: ignore
    from ._models import Datastore  # type: ignore
    from ._models import DatastoreCredentials  # type: ignore
    from ._models import DatastoreSecrets  # type: ignore
    from ._models import DistributionConfiguration  # type: ignore
    from ._models import DockerCredentialDto  # type: ignore
    from ._models import EarlyTerminationPolicy  # type: ignore
    from ._models import EnvironmentContainerData  # type: ignore
    from ._models import EnvironmentContainerDetails  # type: ignore
    from ._models import EnvironmentContainerResourceArmPaginatedResult  # type: ignore
    from ._models import EnvironmentVersionData  # type: ignore
    from ._models import EnvironmentVersionDetails  # type: ignore
    from ._models import EnvironmentVersionResourceArmPaginatedResult  # type: ignore
    from ._models import ErrorAdditionalInfo  # type: ignore
    from ._models import ErrorDetail  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import FlavorData  # type: ignore
    from ._models import IdAssetReference  # type: ignore
    from ._models import IdentityConfiguration  # type: ignore
    from ._models import ImageReferenceForConsumptionDto  # type: ignore
    from ._models import InferenceContainerProperties  # type: ignore
    from ._models import InferencingServer  # type: ignore
    from ._models import IntellectualProperty  # type: ignore
    from ._models import Job  # type: ignore
    from ._models import JobBase  # type: ignore
    from ._models import JobInput  # type: ignore
    from ._models import JobInputDataset  # type: ignore
    from ._models import JobInputLiteral  # type: ignore
    from ._models import JobInputUri  # type: ignore
    from ._models import JobLimits  # type: ignore
    from ._models import JobOutput  # type: ignore
    from ._models import JobOutputDataset  # type: ignore
    from ._models import JobOutputUri  # type: ignore
    from ._models import JobService  # type: ignore
    from ._models import MLTableData  # type: ignore
    from ._models import ManagedIdentity  # type: ignore
    from ._models import ManagedIdentityCredentialDto  # type: ignore
    from ._models import MedianStoppingPolicy  # type: ignore
    from ._models import ModelConfiguration  # type: ignore
    from ._models import ModelContainerData  # type: ignore
    from ._models import ModelContainerDetails  # type: ignore
    from ._models import ModelContainerResourceArmPaginatedResult  # type: ignore
    from ._models import ModelPackageInput  # type: ignore
    from ._models import ModelVersionData  # type: ignore
    from ._models import ModelVersionDetails  # type: ignore
    from ._models import ModelVersionResourceArmPaginatedResult  # type: ignore
    from ._models import Mpi  # type: ignore
    from ._models import NoneDatastoreCredentials  # type: ignore
    from ._models import Objective  # type: ignore
    from ._models import OnlineInferenceConfiguration  # type: ignore
    from ._models import OutputPathAssetReference  # type: ignore
    from ._models import PackageInputPathBase  # type: ignore
    from ._models import PackageInputPathId  # type: ignore
    from ._models import PackageInputPathUrl  # type: ignore
    from ._models import PackageInputPathVersion  # type: ignore
    from ._models import PackageRequest  # type: ignore
    from ._models import PackageResponse  # type: ignore
    from ._models import PipelineJob  # type: ignore
    from ._models import PyTorch  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceBase  # type: ignore
    from ._models import ResourceConfiguration  # type: ignore
    from ._models import ResourceManagementAssetReferenceData  # type: ignore
    from ._models import ResourceManagementAssetReferenceDetails  # type: ignore
    from ._models import Route  # type: ignore
    from ._models import SASCredentialDto  # type: ignore
    from ._models import SasDatastoreCredentials  # type: ignore
    from ._models import SasDatastoreSecrets  # type: ignore
    from ._models import ServicePrincipalDatastoreCredentials  # type: ignore
    from ._models import ServicePrincipalDatastoreSecrets  # type: ignore
    from ._models import SweepJob  # type: ignore
    from ._models import SweepJobLimits  # type: ignore
    from ._models import SystemData  # type: ignore
    from ._models import TemporaryDataReferenceRequestDto  # type: ignore
    from ._models import TemporaryDataReferenceResponseDto  # type: ignore
    from ._models import TensorFlow  # type: ignore
    from ._models import TrialComponent  # type: ignore
    from ._models import TritonInferencingServer  # type: ignore
    from ._models import TruncationSelectionPolicy  # type: ignore
    from ._models import UriFileDataVersion  # type: ignore
    from ._models import UriFolderDataVersion  # type: ignore
    from ._models import UriReference  # type: ignore

from ._azure_machine_learning_workspaces_enums import (
    BaseEnvironmentSourceType,
    BindingType,
    CreatedByType,
    CredentialsType,
    DataReferenceCredentialType,
    DataType,
    DatastoreType,
    DistributionType,
    EarlyTerminationPolicyType,
    EnvironmentType,
    Goal,
    IdentityConfigurationType,
    InferencingServerType,
    InputDataDeliveryMode,
    InputPathType,
    JobInputType,
    JobLimitsType,
    JobOutputType,
    JobStatus,
    JobType,
    ListViewType,
    OperatingSystemType,
    OutputDataDeliveryMode,
    PackageBuildState,
    PackageInputDeliveryMode,
    PackageInputType,
    ProtectionLevel,
    ReferenceType,
    SamplingAlgorithm,
    SecretsType,
    ServiceDataAccessAuthIdentity,
)

__all__ = [
    "AccountKeyDatastoreCredentials",
    "AccountKeyDatastoreSecrets",
    "AcrDetail",
    "AmlToken",
    "AssetBase",
    "AssetContainer",
    "AssetReferenceBase",
    "AzureBlobDatastore",
    "AzureDataLakeGen1Datastore",
    "AzureDataLakeGen2Datastore",
    "AzureFileDatastore",
    "AzureMLBatchInferencingServer",
    "AzureMLOnlineInferencingServer",
    "BanditPolicy",
    "BaseEnvironmentId",
    "BaseEnvironmentSource",
    "BasicBinding",
    "Binding",
    "BlobReferenceForConsumptionDto",
    "BlobReferenceSASRequestDto",
    "BlobReferenceSASResponseDto",
    "BuildContext",
    "CertificateDatastoreCredentials",
    "CertificateDatastoreSecrets",
    "CodeConfiguration",
    "CodeContainerData",
    "CodeContainerDetails",
    "CodeContainerResourceArmPaginatedResult",
    "CodeVersionData",
    "CodeVersionDetails",
    "CodeVersionResourceArmPaginatedResult",
    "CommandJob",
    "CommandJobLimits",
    "ComponentContainerData",
    "ComponentContainerDetails",
    "ComponentContainerResourceArmPaginatedResult",
    "ComponentJob",
    "ComponentVersionData",
    "ComponentVersionDetails",
    "ComponentVersionResourceArmPaginatedResult",
    "CustomInferencingServer",
    "DataContainerData",
    "DataContainerDetails",
    "DataContainerResourceArmPaginatedResult",
    "DataPathAssetReference",
    "DataReferenceCredentialDto",
    "DataVersionBaseData",
    "DataVersionBaseDetails",
    "DataVersionBaseResourceArmPaginatedResult",
    "Datastore",
    "DatastoreCredentials",
    "DatastoreSecrets",
    "DistributionConfiguration",
    "DockerCredentialDto",
    "EarlyTerminationPolicy",
    "EnvironmentContainerData",
    "EnvironmentContainerDetails",
    "EnvironmentContainerResourceArmPaginatedResult",
    "EnvironmentVersionData",
    "EnvironmentVersionDetails",
    "EnvironmentVersionResourceArmPaginatedResult",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "FlavorData",
    "IdAssetReference",
    "IdentityConfiguration",
    "ImageReferenceForConsumptionDto",
    "InferenceContainerProperties",
    "InferencingServer",
    "IntellectualProperty",
    "Job",
    "JobBase",
    "JobInput",
    "JobInputDataset",
    "JobInputLiteral",
    "JobInputUri",
    "JobLimits",
    "JobOutput",
    "JobOutputDataset",
    "JobOutputUri",
    "JobService",
    "MLTableData",
    "ManagedIdentity",
    "ManagedIdentityCredentialDto",
    "MedianStoppingPolicy",
    "ModelConfiguration",
    "ModelContainerData",
    "ModelContainerDetails",
    "ModelContainerResourceArmPaginatedResult",
    "ModelPackageInput",
    "ModelVersionData",
    "ModelVersionDetails",
    "ModelVersionResourceArmPaginatedResult",
    "Mpi",
    "NoneDatastoreCredentials",
    "Objective",
    "OnlineInferenceConfiguration",
    "OutputPathAssetReference",
    "PackageInputPathBase",
    "PackageInputPathId",
    "PackageInputPathUrl",
    "PackageInputPathVersion",
    "PackageRequest",
    "PackageResponse",
    "PipelineJob",
    "PyTorch",
    "Resource",
    "ResourceBase",
    "ResourceConfiguration",
    "ResourceManagementAssetReferenceData",
    "ResourceManagementAssetReferenceDetails",
    "Route",
    "SASCredentialDto",
    "SasDatastoreCredentials",
    "SasDatastoreSecrets",
    "ServicePrincipalDatastoreCredentials",
    "ServicePrincipalDatastoreSecrets",
    "SweepJob",
    "SweepJobLimits",
    "SystemData",
    "TemporaryDataReferenceRequestDto",
    "TemporaryDataReferenceResponseDto",
    "TensorFlow",
    "TrialComponent",
    "TritonInferencingServer",
    "TruncationSelectionPolicy",
    "UriFileDataVersion",
    "UriFolderDataVersion",
    "UriReference",
    "BaseEnvironmentSourceType",
    "BindingType",
    "CreatedByType",
    "CredentialsType",
    "DataReferenceCredentialType",
    "DataType",
    "DatastoreType",
    "DistributionType",
    "EarlyTerminationPolicyType",
    "EnvironmentType",
    "Goal",
    "IdentityConfigurationType",
    "InferencingServerType",
    "InputDataDeliveryMode",
    "InputPathType",
    "JobInputType",
    "JobLimitsType",
    "JobOutputType",
    "JobStatus",
    "JobType",
    "ListViewType",
    "OperatingSystemType",
    "OutputDataDeliveryMode",
    "PackageBuildState",
    "PackageInputDeliveryMode",
    "PackageInputType",
    "ProtectionLevel",
    "ReferenceType",
    "SamplingAlgorithm",
    "SecretsType",
    "ServiceDataAccessAuthIdentity",
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

__path__ = __import__("pkgutil").extend_path(__path__, __name__)

from .azure_openai_finetuning import AzureOpenAIFineTuningSchema
from .azure_openai_hyperparameters import AzureOpenAIHyperparametersSchema
from .custom_model_finetuning import CustomModelFineTuningSchema
from .finetuning_job import FineTuningJobSchema
from .finetuning_vertical import FineTuningVerticalSchema

__all__ = [
    "AzureOpenAIFineTuningSchema",
    "AzureOpenAIHyperparametersSchema",
    "CustomModelFineTuningSchema",
    "FineTuningJobSchema",
    "FineTuningVerticalSchema",
]



# coding: utf-8
import json

import lightgbm as lgb
import numpy as np
import pandas as pd
from mlflow.models.signature import infer_signature
from sklearn.metrics import mean_squared_error

try:
    import cPickle as pickle
except BaseException:
    import pickle

import argparse

import mlflow

# mlflow.lightgbm.autolog()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--lr", default=0.05)
    parser.add_argument("--feature_fraction", default=0.9)
    parser.add_argument("--bagging_fraction", default=0.8)
    args = parser.parse_args()

    print("Loading data...")
    # load or create your dataset
    df_train = pd.read_csv("./binary_classification/binary.train", header=None, sep="\t")
    df_test = pd.read_csv("./binary_classification/binary.test", header=None, sep="\t")
    W_train = pd.read_csv("./binary_classification/binary.train.weight", header=None)[0]
    W_test = pd.read_csv("./binary_classification/binary.test.weight", header=None)[0]

    y_train = df_train[0]
    y_test = df_test[0]
    X_train = df_train.drop(0, axis=1)
    X_test = df_test.drop(0, axis=1)

    num_train, num_feature = X_train.shape

    # create dataset for lightgbm
    # if you want to re-use data, remember to set free_raw_data=False
    lgb_train = lgb.Dataset(X_train, y_train, weight=W_train, free_raw_data=False)
    lgb_eval = lgb.Dataset(X_test, y_test, reference=lgb_train, weight=W_test, free_raw_data=False)

    # specify your configurations as a dict
    params = {
        "boosting_type": "gbdt",
        "objective": "binary",
        "metric": "binary_logloss",
        "num_leaves": 31,
        "learning_rate": args.lr,
        "feature_fraction": args.feature_fraction,
        "bagging_fraction": args.bagging_fraction,
        "bagging_freq": 5,
        "verbose": 0,
    }

    # generate feature names
    feature_name = ["feature_" + str(col) for col in range(num_feature)]

    print("Starting training...")
    # feature_name and categorical_feature
    gbm = lgb.train(
        params,
        lgb_train,
        num_boost_round=10,
        valid_sets=[lgb_train],  # eval training data
        valid_names="train",
        feature_name=feature_name,
        early_stopping_rounds=10,
        categorical_feature=[21],
    )

    print("Finished first 10 rounds...")
    # check feature name
    print("7th feature name is:", lgb_train.feature_name[6])

    print("Saving model...")
    # save model to file
    gbm.save_model("model.txt")

    print("Dumping model to JSON...")
    # dump model to JSON (and save to file)
    model_json = gbm.dump_model()

    with open("model.json", "w+") as f:
        json.dump(model_json, f, indent=4)

    # feature names
    print("Feature names:", gbm.feature_name())

    # feature importances
    print("Feature importances:", list(gbm.feature_importance()))

    print("Loading model to predict...")
    # load model to predict
    bst = lgb.Booster(model_file="model.txt")
    # can only predict with the best iteration (or the saving iteration)
    y_pred = bst.predict(X_test)
    # eval with loaded model
    print("The rmse of loaded model's prediction is:", mean_squared_error(y_test, y_pred) ** 0.5)

    print("Dumping and loading model with pickle...")
    # dump model with pickle
    with open("model.pkl", "wb") as fout:
        pickle.dump(gbm, fout)
    # load model with pickle to predict
    with open("model.pkl", "rb") as fin:
        pkl_bst = pickle.load(fin)
    # can predict with any iteration when loaded in pickle way
    y_pred = pkl_bst.predict(X_test, num_iteration=7)
    # eval with loaded model
    print("The rmse of pickled model's prediction is:", mean_squared_error(y_test, y_pred) ** 0.5)

    # continue training
    # init_model accepts:
    # 1. model file name
    # 2. Booster()
    gbm = lgb.train(params, lgb_train, num_boost_round=10, init_model="model.txt", valid_sets=lgb_eval)

    print("Finished 10 - 20 rounds with model file...")

    # decay learning rates
    # learning_rates accepts:
    # 1. list/tuple with length = num_boost_round
    # 2. function(curr_iter)
    gbm = lgb.train(
        params,
        lgb_train,
        num_boost_round=10,
        init_model=gbm,
        learning_rates=lambda iter: 0.05 * (0.99**iter),
        valid_sets=lgb_eval,
    )

    print("Finished 20 - 30 rounds with decay learning rates...")

    # change other parameters during training
    gbm = lgb.train(
        params,
        lgb_train,
        num_boost_round=10,
        init_model=gbm,
        valid_sets=lgb_eval,
        callbacks=[lgb.reset_parameter(bagging_fraction=[0.7] * 5 + [0.6] * 5)],
    )

    print("Finished 30 - 40 rounds with changing bagging_fraction...")

    # self-defined objective function
    # f(preds: array, train_data: Dataset) -> grad: array, hess: array
    # log likelihood loss
    def loglikelihood(preds, train_data):
        labels = train_data.get_label()
        preds = 1.0 / (1.0 + np.exp(-preds))
        grad = preds - labels
        hess = preds * (1.0 - preds)
        return grad, hess

    # self-defined eval metric
    # f(preds: array, train_data: Dataset) -> name: string, eval_result: float, is_higher_better: bool
    # binary error
    # NOTE: when you do customized loss function, the default prediction value is margin
    # This may make built-in evalution metric calculate wrong results
    # For example, we are doing log likelihood loss, the prediction is score before logistic transformation
    # Keep this in mind when you use the customization
    def binary_error(preds, train_data):
        labels = train_data.get_label()
        preds = 1.0 / (1.0 + np.exp(-preds))
        return "error", np.mean(labels != (preds > 0.5)), False

    gbm = lgb.train(
        params,
        lgb_train,
        num_boost_round=10,
        init_model=gbm,
        fobj=loglikelihood,
        feval=binary_error,
        valid_sets=lgb_eval,
    )

    print("Finished 40 - 50 rounds with self-defined objective function and eval metric...")

    input_sample = X_train.head(1)
    signature = infer_signature(input_sample, y_pred)

    mlflow.lightgbm.log_model(
        gbm,
        "lightgbm_artifact",
        signature=signature,
        input_example=input_sample,
        registered_model_name="lightgbm_predict",
    )

    # another self-defined eval metric
    # f(preds: array, train_data: Dataset) -> name: string, eval_result: float, is_higher_better: bool
    # accuracy
    # NOTE: when you do customized loss function, the default prediction value is margin
    # This may make built-in evalution metric calculate wrong results
    # For example, we are doing log likelihood loss, the prediction is score before logistic transformation
    # Keep this in mind when you use the customization
    def accuracy(preds, train_data):
        labels = train_data.get_label()
        preds = 1.0 / (1.0 + np.exp(-preds))
        return "accuracy", np.mean(labels == (preds > 0.5)), True


if __name__ == "__main__":
    main()
